@@ -27,14 +27,24 @@ _m = []
 
 class Word(type):
     """
-    When Namable is extended, this adds 1 defclass to clips
+    When Namable is extended,
+    __init__ adds 1 defclass to clips
     creating a subclass of Namable.
-    And registers the class in subclasses
+    And registers the class in utils.subclasses.
+    If utils.varpat matches the classname,
+    however, __new__ builds a ClassVar intance
+    and returns it, thus bypassing __init__.
+    In that case, bases can be a subclass of
+    State or Thing.
     """
-    def __new__(cls, classname, bases, newdict=None):
+    def __new__(cls, classname, bases=None, newdict=None):
 
         if utils.varpat.match(classname):
-            return ClassVar(classname, bases)
+            if bases:
+                return ClassVar(classname, bases)
+            elif getattr(cls, 'cls', False):
+                return ClassVar(classname, cls.cls)
+            return ClassVar(classname, utils.get_class('Namable'))
         cls.value = ''
         return super(Word, cls).__new__(cls, classname, bases, newdict)
 
@@ -46,21 +56,42 @@ class Word(type):
         utils.register(classname, cls)
 
     def clsput(self, vrs, ancestor=None, mod_path=None):
+        '''
+        return clips snippet to be used
+        when the class is the subject in a fact
+        or a mod in a predicate
+        and the sentence is being asserted
+        or is in the head of a rule.
+        Also when the class is used as subject
+        of a fact in the tail of a rule.
+        '''
         return self.__name__
 
     def get_constraint_cls(self, vrs, ancestor=None, mod_path=None):
+        '''
+        return clips snippet to be used
+        when the class is a mod in a predicate
+        and the sentence is in the tail of a rule.
+        '''
         ci = utils.clips_instance(ancestor, mod_path)
         return '&:(eq %s %s)' % (self.__name__, ci)
 
     def get_isc_cls(self, templs, queries, vrs):
         """
-        get instance-set condition;
+        build clips snippet
+        to be used when the class is the subject in a fact
+        or a mod in a predicate
+        and the sentence is in a query.
         return (instance-set templates, instance-set queries)
         """
         return self.__name__
 
     @classmethod
     def from_clips(cls, instance):
+        '''
+        build nl instance starting from a clips instance
+        or instance name
+        '''
         if not isinstance(instance, clips._clips_wrap.Class):
             instance = clips.FindClass(instance)
         clsname = str(instance.Name)
@@ -70,20 +101,34 @@ utils.register('Word', Word)
 
 class ClassVar(object):
     '''
-    falta registrar noun y verb
-    y dar los métodos __init__, get_slot_constraint,
-    y un __call__ que construya un objeto de la clase apropiada.
-    y claro, hace falta que las clases (es decir, noun y verb)
-    tengan los métodos get_slot_constraint etc.
+    Used in rules, in the head or tail,
+    as subject or mod in a predicate,
+    when Word is called with a string
+    that matches varpat.
+    Intances can be called, and give back
+    an instance of ClassVarVar,
+    that can handle clips for
+    the case in which we have both
+    the class and the name or mods
+    as variables.
+
+    This is not to be part of the public api
     '''
-    def __init__(self, var, cls):
+    def __init__(self, var, cls=None):
         self.value = var
-        self.cls = cls
+        self.cls = cls and cls or utils.get_class('Namable')
 
     def __call__(self, var='', **kwargs):
-        return ClassVarVar(self.value, self.cls, var, **kwargs)
+        if utils.varpat.match(self.value):
+            return ClassVarVar(self.value, self.cls, var, **kwargs)
+        return self.cls(self.value)
 
     def get_constraint(self, vrs, ancestor, mod_path):
+        '''
+        return clips snippet to be used
+        when the class var is a mod in a predicate
+        and the sentence is in the tail of a rule.
+        '''
         ci = utils.clips_instance(ancestor, mod_path)
         if self.value in vrs:
             return self.cls(self.value).get_constraint(vrs, ancestor, mod_path)
@@ -96,8 +141,9 @@ class ClassVar(object):
 
     def get_slot_constraint(self, vrs):
         """
-        build rule CE constraint for clips
-        for a slot constraint for a pred in a prop in a rule
+        return clips snippet to be used
+        when the class var is predicate or subject
+        and the sentence is in the tail of a rule.
         """
         if self.value in vrs:
             return self.cls(self.value).get_var_slot_constraint(vrs, self.value)
@@ -110,7 +156,10 @@ class ClassVar(object):
 
     def get_isc(self, templs, queries, vrs):
         """
-        get instance-set condition;
+        build clips snippet
+        to be used when the class var is the subject in a fact
+        or a mod in a predicate
+        and the sentence is in a query.
         return (instance-set templates, instance-set queries)
         """
         if self.value in vrs and vrs[self.value]:
@@ -126,11 +175,20 @@ class ClassVar(object):
         return '?%s' % self.value
 
     def put(self, vrs):
+        # a hack
         return self.cls(self.value).put(vrs)
 
 
 class ClassVarVar(object):
     '''
+    Intances of ClasVar can be called, and give back
+    an instance of ClassVarVar,
+    that can handle clips for
+    the case in which we have both
+    the class and the name or mods
+    as variables.
+
+    This is not to be part of the public api
     '''
     def __init__(self, clsvar, cls, var, **kwargs):
         self.value = var
@@ -139,6 +197,11 @@ class ClassVarVar(object):
         self.ob = cls(var, **kwargs)
 
     def get_constraint(self, vrs, ancestor, mod_path):
+        '''
+        return clips snippet to be used
+        when the variable is a mod in a predicate
+        and the sentence is in the tail of a rule.
+        '''
         ci = utils.clips_instance(ancestor, mod_path)
         constraint = []
         if not self.value or self.value in vrs:
@@ -154,8 +217,9 @@ class ClassVarVar(object):
 
     def get_slot_constraint(self, vrs):
         """
-        build rule CE constraint for clips
-        for a slot constraint for a pred in a prop in a rule
+        return clips snippet to be used
+        when the variable is predicate or subject
+        and the sentence is in the tail of a rule.
         """
         if not self.value or self.value in vrs:
             return self.ob.get_var_slot_constraint(vrs, self.value)
@@ -187,12 +251,43 @@ class ClassVarVar(object):
         return '?%s' % self.value
 
 
+class Subword(object):
+    '''
+    Instantiated with two subclasses of Namable,
+    Or of ClassVar created with Word(var),
+    it can be used as a premise in a rule.
+    '''
+    def __init__(self, sub, sup):
+        self.sub = sub
+        self.sup = sup
+
+    def get_ce(self, vrs):
+        '''
+        '''
+        sub = getattr(self.sub, 'clsput', self.sub.put)(vrs)
+        sup = getattr(self.sup, 'clsput', self.sup.put)(vrs)
+        return ''' (test (or (eq %(sub)s %(sup)s)
+                             (subclassp %(sub)s %(sup)s)))
+               ''' % {'sub': sub,
+                      'sup': sup}
+
+
 class Noun(Word):
     """
-    When Namable is extended, this adds 1 defclass to clips
+    When Thing is extended, this adds 1 defclass to clips
     creating a subclass of Namable.
-    And registers the class in subclasses
+    And registers the class in utils.subclasses.
+
+    If utils.varpat matches classname,
+    it sets cls to Thing so that Word.__new__
+    can return the right ClassVar,
+    unless bases is a Thing subclass.
     """
+    def __new__(cls, classname, bases=None, newdict=None):
+        if utils.varpat.match(classname) and not bases:
+            cls.cls = utils.get_class('Thing')
+        return Word.__new__(cls, classname, bases, newdict)
+
     def __init__(cls, classname, bases, newdict):
         superclassname = bases[0].__name__
         clp = '(defclass %s (is-a %s))' % (classname, bases[0].__name__)
@@ -204,8 +299,24 @@ utils.register('Noun', Noun)
 
 class Verb(Word):
     """
-    When State is extended, this registers the class in _subclasses
+    When State is extended, this adds 1 defclass to clips
+    creating a subclass of Namable.
+    And registers the class in utils.subclasses.
+
+    The new class mods dictionary is also
+    initialized with the given mods
+    and the mods of all its bases.
+
+    If utils.varpat matches classname,
+    it sets cls to State so that Word.__new__
+    can return the right ClassVar,
+    unless bases is a State subclass.
     """
+    def __new__(cls, classname, bases=None, newdict=None):
+        if utils.varpat.match(classname) and not bases:
+            cls.cls = utils.get_class('State')
+        return Word.__new__(cls, classname, bases, newdict)
+
     def __init__(cls, classname, bases, newdict):
         if classname == 'State':
             # due to the ordering of things in clps.py
@@ -233,6 +344,8 @@ utils.register('Verb', Verb)
 
 class Namable(object):
     """
+    abstract class ancestor of all excluding
+    the metawords.
     """
     __metaclass__ = Word
 
@@ -280,7 +393,21 @@ class Namable(object):
 
 class Number(Namable):
     """
+    A number or an arithmetic operation.
+    As value it can take a number, a str(number),
+    a string containing a single arithmetic operator,
+    within ['+', '-', '*', '/', '**']
+    or a string with an arithmetic operation
+    of the form '(<op> <arg1> <arg2>)'
+    where op is one of the operators
+    and arg can be a number or another operation.
+    If the value is an operator,
+    the args are either instances of Number,
+    or strings to be converted in such instances.
 
+    Within rules, variables can take the place of
+    value, any of the args, or in the value in the position
+    of numbers when it is a string representing an operation.
     """
     def __init__(self, value, arg1='', arg2=''):
         self.arg1 = arg1
@@ -344,7 +471,16 @@ class Number(Namable):
 
 class Arith(Number):
     """
-    Arithmetic predicate
+    Arithmetic predicate.
+    It has the same basic form as number
+    except that value can only be, either
+    a string with an arithmetic predicate symbol
+    from ['=', '<=', '>=', '!=', '<', '>'],
+    or a string with a predication of the same form
+    as the strings for number, but with the outermost
+    operator substituted for a predicate.
+
+    Instances can only be used as premises in rules.
     """
     def __init__(self, value, arg1='', arg2=''):
         if value[0] == '(':
