@@ -34,6 +34,12 @@ _m = []
 
 class Fact(Namable):
     """
+    A fact is built with a subj and a pred positional arguments.
+    subj is either an object of Thing or any of its subclasses,
+    or a class chosen among Thing, State, or any subclass of them.
+    It can also take a named argument, often given as a 3rd
+    positional arg, that has to be of type Time,
+    and a named arg 'truth', that is either 0 or 1.
     """
     def __init__(self, subj, pred, time=Instant('now'), truth=1):
         self.truth = truth
@@ -125,3 +131,19 @@ class Fact(Namable):
         t = self.time.put(vrs)
         return '(add-prop %s %s %s %s)' % (s, p, t, self.truth)
 
+
+
+def factback(csubj, cpred, ctime, ctruth):
+    """
+    """
+    try:
+        subj = Namable.from_clips(csubj)
+    except clips.ClipsError:
+        subj = Word.from_clips(csubj)
+    pred = Namable.from_clips(cpred)
+    time = Time.from_clips(ctime)
+    truth = int(str(ctruth))
+    fact = Fact(subj, pred, time, truth=truth)
+    pred.in_fact(fact)
+
+clips.RegisterPythonFunction(factback)
